// ***************************************************************************
//        First-return map to a Poincaré section of the Rössler system
//
//                                       Christophe Letellier, October 2, 2019
// ***************************************************************************
# include <cstdlib>
# include <iostream>
# include <fstream>
# include <iomanip>
# include <cmath>
# include <ctime>

using namespace std;

  double a;
  const double b = 2.0;
  const double c = 4.0;

 double *rossler (int m,double x[]);
 double *rk4vec (int n, double u0[], double dt, double *f(int n, double u[]) );

int main ( )
{
  int i, j, k, px;
  const int m = 3;                                     // state space dimension 
  const int Ntrans = 10000;
  const int Npt = 100;                                 // number of data points
  const int Nbif = 500;                           // number of parameter values
  const int Nsec = 300;             // Number of pixels in the Poincaré section 
  const double dt = 0.01;                                          // Time step
  const double amin = 0.30;            // Interval of the bifurcation parameter
  const double amax = 0.55;
  const double da=(amax-amin)/double(Nbif);
  const double Ymin = -7.0;                 // Interval of the Poincaré section
  const double Ymax = 0.0;
  const double dY=(Ymax-Ymin)/double(Nsec);
  double xp,yp,zp,sx,sy,sz; 
  double *x; 
  bool *gate;

  x = new double[m];
  gate = new bool[Nsec];

  ofstream data_12 ("rosbif.dat");

  x[0]=0.3;                                               // Initial conditions
  x[1]=0.3;
  x[2]=0.1;
  for (a = amin; a <= amax; a+=da)
  {
    cout << a << "\n";
    xp = (c-sqrt(c*c-4.0*a*b))/2.0;         // Definition of the Poincaré plane
    for (px = 0; px <= Nsec-1; px++)
    {
      gate[px] = true;
    }
    j = 0;
    k = 0;
    do 
    {
      sx = x[0];
      sy = x[1];
      sz = x[2];
      x=rk4vec (m,x,dt,rossler);
      j++;
      if (j > Ntrans) 
      {
        if ((x[0] >= xp) && (sx < xp))
        {
          k++;
          yp = x[1]-((x[0]-xp)/(x[0]-sx))*(x[1]-sy);
          px = int(round((yp-Ymin)/dY));
          if (gate[px])
          {
            gate[px] = false;
            data_12 << a << " " << yp << "\n";
          }
        }
      }
    } while (k < Npt);
  }
  data_12.close ( );
  system("xmgrace rosbif.dat -param rosbif.par"); 
  return 0;
}
// **************************************************************************
 double *rossler (int m,double x[])
// **************************************************************************
{
  double *dxdt;

  dxdt = new double[m];

  dxdt[0] = -x[1]-x[2];
  dxdt[1] = x[0]+a*x[1];
  dxdt[2] = b+x[2]*(x[0]-c);
  return dxdt;
}
//***************************************************************************
 double *rk4vec (int m, double u0[], double dt, double *f(int m,double u[]) )
//***************************************************************************
//    09 October 2013 John Burkardt
//
//    Input, int m, the spatial dimension.
//    Input, double U0[m], the solution estimate at the current time.
//    Input, double DT, the time step.
//    Input, double *F (int m, double U[])
//    Output, double RK4VEC[M], the fourth-order Runge-Kutta solution estimate
{
  int i;
  double *f0; double *f1; double *f2; double *f3;
  double *u; double *u1; double *u2; double *u3;
  f0=f(m,u0);
  u1=new double[m];
  for (i=0; i<m; i++)
  {
    u1[i]=u0[i]+dt*f0[i]/2.0;
  }
  f1=f(m,u1);
  u2=new double[m];
  for (i=0; i<m; i++)
  {
    u2[i]=u0[i]+dt*f1[i]/2.0;
  }
  f2=f(m,u2);
  u3=new double[m];
  for (i=0; i<m; i++)
  {
     u3[i]=u0[i]+dt*f2[i];
  }
  f3=f(m,u3);
//  Combine them to estimate the solution.
  u=new double[m];
  for (i=0; i<m; i++)
  {
     u[i]=u0[i]+dt*(f0[i]+2.0*f1[i]+2.0*f2[i]+f3[i])/6.0;
  }
//  Free memory.
  delete [] f0;
  delete [] f1;
  delete [] f2;
  delete [] f3;
  delete [] u1;
  delete [] u2;
  delete [] u3;
  return u;
}
